/* Decode and fill a memory buffer with images obtained from an AVI  */
/* video stream. The file is structured to be compatible with the    */
/* FLC/FLI reading and decoding functions                            */

//
// The basic strategy used by this file is to open the AVI file
// find the first video stream and record a pointer to it.
// Each time the next image is required the API function
// "AVIStreamGetFrame( )" copies the information into a memory buffer
// that is organized in the form of a Device Independent Bitmap using
// an 8 bit (256 color) format.  The data organization of a DIB is not
// the same as that required by a DirectDraw Surface (pointer to by
// "*Screen". Therefore the function "CopyToScreen()" is uses to transfer
// the image to the DirectDraw Surface.
//

#define  STRICT
#include <stdio.h>
#include <windows.h>
#include <mmsystem.h>
#include <vfw.h>

#define FLC_type 0
#define FLI_type 1
#define AVI_type 2

#define BUFSIZE 256

//
// Global variables.
//

extern unsigned char p_red[],p_green[],p_blue[];   // Palette
extern unsigned char *Screen;                      // Surface
extern long pitch;                                 // Surface memory stride
extern long AnimationType;

static LPBITMAPINFOHEADER lpBmi = NULL;
static LPSTR pformat            = NULL;
static PAVIFILE pfile           = NULL;
static PAVISTREAM ps            = NULL;
static PGETFRAME  pget          = NULL;
static long display_sample      = 0;
static long nsamples;

static void CopyToScreen(void);   // Local function

void BlitFirstAviFrame(char *szFileName){
 if(szFileName != NULL){
   AVISTREAMINFO avis;
   HRESULT hr;
   RGBQUAD *pRgb;
   int i,j;
   long size;
   AVIFileInit();
   //
   // Open the AVI file
   //
   hr = AVIFileOpen(&pfile,szFileName,OF_READ,NULL);
   if(hr == AVIERR_OK){
     for(i=0;i<10;i++){
       //
       // Get the first Video stream
       //
       if(AVIFileGetStream(pfile,&ps,0,i) == AVIERR_OK){
         AVIStreamInfo(ps,&avis,sizeof(avis));
         if(avis.fccType == streamtypeVIDEO){
           //
           // This is a video stream
           //
           AVIStreamFormatSize(ps,0,&size);
           if((pformat = (LPSTR)malloc(size)) != NULL){
             //
             // Get the format of the stream - most importantly
             // the number of frames in the animation.
             //
             if(AVIStreamReadFormat(ps,0,pformat,&size) == AVIERR_OK){
               lpBmi=(LPBITMAPINFOHEADER)pformat;
               nsamples=AVIStreamEnd(ps);     // number of frames in stream
               //
               // Before reading the image data specify the format
               // in which the image data should be returned. In this
               // program we need an 8 bit paletted image (Not an RGB one!).
               //
               lpBmi->biCompression=BI_RGB;  // read it back as 8 bit
               lpBmi->biBitCount=8;          // paletted image
               lpBmi->biClrUsed=256;         //
               lpBmi->biSizeImage=((((UINT)lpBmi->biBitCount*lpBmi->biWidth
                                  +31)&~31)/8)*lpBmi->biHeight;
               //
               // Open the video stream using the format needed
               //
               if((pget=AVIStreamGetFrameOpen(ps,lpBmi)) != NULL){
                 //
                 // Retrieve the image
                 //
                 lpBmi=(LPBITMAPINFOHEADER)AVIStreamGetFrame(pget,0);
                 if(lpBmi != NULL){
                   //
                   // Extract the palette information
                   //
                   pRgb=(RGBQUAD *)((LPSTR)lpBmi + lpBmi->biSize);
                   for(j=0;j<256;j++){
                     p_red[j]   = pRgb[j].rgbRed;
                     p_green[j] = pRgb[j].rgbGreen;
                     p_blue[j]  = pRgb[j].rgbBlue;
                   }
                   //
                   // Copy the image data from temporary DIB to
                   // DirectDraw surface.
                   //
                   CopyToScreen();
                   //
                   // Keep the stream open and memory reserved for
                   // DIB.
                   //
                   return;
                 }
               }
             }
           }
         }
       }
     }
   }
   else pfile=NULL;
 }
 return;
}

void BlitNextAviFrame(void){
 //
 // Read the next frame (called the "sample"). Remember that the
 // video stream and file remain open.
 //
 lpBmi=(LPBITMAPINFOHEADER)AVIStreamGetFrame(pget,display_sample);
 if(lpBmi != NULL)CopyToScreen();
 //
 // If we are about to go beyond the end of the video stream reset
 // the frame count so that the first frame is loaded next time.
 //
 if(++display_sample >= nsamples)display_sample=0;
}

void CloseAvi(void){
 //
 // Close all the streams files and release temporary memory.
 //
 if(pget != NULL)AVIStreamGetFrameClose(pget);
 if(pformat != NULL)free(pformat);
 if(pfile != NULL)AVIFileRelease(pfile);
 AVIFileExit();
 lpBmi   = NULL;
 pformat = NULL;
 pfile   = NULL;
 ps      = NULL;
 pget    = NULL;
}

static void CopyToScreen(void){
 LPSTR lpB,s;
 long j,x,y;
 lpB =  (LPSTR)(lpBmi);
 lpB += (lpBmi->biSize+256*sizeof(RGBQUAD));  // points to Image
 x=(long)lpBmi->biWidth;
 y=(long)lpBmi->biHeight;
 //
 // Copy each display line from DIB to screen. A DIB is upside down
 // in relation to the screen in which the rows start at the top.
 // DirectDraw specifies that each raster line is separated by "pitch"
 // bytes (number of bytes). This might or might NOT be the same as
 // the horizontal resolution of the display. For this reason it is
 // important to copy one row at a time and increment the "Screen"
 // pointer by "pitch" for each row.
 //
 for(j=0;j<y;j++){
   s=(LPSTR)Screen+(y-j-1)*pitch;     // s -> start of raster row "j"
   memcpy(s,lpB,x);
   lpB += x;                          // lpB -> next DIB row
 }
 return;
}
