/*=========================================================================

  Program:   Visualization Toolkit
  Module:    vtkObject.hh


=========================================================================*/
// .NAME vtkObject - abstract base class for most of the vtk objects
// .SECTION Description
// vtkObject is the base class for many objects in the visualization 
// toolkit. vtkObject provides methods for tracking modification times, 
// debugging, and printing. Most objects created within the vtk 
// framework should be a subclass of vtkObject or one of its children.
// The few exceptions tend to be very small helper classes that usually
// never get instantiated or situations where multiple inheritance
// gets in the way.  Then LWObject might be required.

// .SECTION see also
// vtkLWObject

#ifndef __vtkObject_hh
#define __vtkObject_hh

#include <iostream.h>
#include "vtkTimeStamp.hh"
#include "vtkSetGet.hh"
#include "vtkIndent.hh"

class vtkObject 
{
public:
  vtkObject(); //create a vtk object
  virtual void Delete(); //delete a vtk object.
  virtual ~vtkObject(); //use Delete() whenever possible
  virtual char *GetClassName() {return "vtkObject";};

  // debugging
  virtual void DebugOn();
  virtual void DebugOff();
  int GetDebug();

  // modified time
  virtual unsigned long int GetMTime();
  virtual void Modified();

  // printing
  virtual void PrintSelf(ostream& os, vtkIndent indent);
  void Print(ostream& os);
  virtual void PrintHeader(ostream& os, vtkIndent indent);
  virtual void PrintTrailer(ostream& os, vtkIndent indent);

protected:
  int Debug;         // Enable debug messages
  vtkTimeStamp MTime; // Keep track of modification time

private:
  friend ostream& operator<<(ostream& os, vtkObject& o);
};

// Description:
// Update the modification time for this object. Many filters rely on
// the modification time to determine if they need to recompute their
// data.
inline void vtkObject::Modified()
{
  this->MTime.Modified();
}

#endif

